@@ -152,20 +152,22 @@ sub process {
     # loop over the parsed elements
     foreach my $el ($self->document->elements) {
         if ($el->type eq 'null') {
+            push @pieces, $self->format_anchors($el) if $el->anchors;
             next;
         }
         if ($el->type eq 'startblock') {
             die "startblock with string passed!: " . $el->string if $el->string;
-            push @pieces, $self->blkstring(start => $el->block, start_list_index => $el->start_list_index);
+            push @pieces, $self->blkstring(start => $el->block, start_list_index => $el->start_list_index),
+              $self->format_anchors($el);
         }
         elsif ($el->type eq 'stopblock') {
             die "stopblock with string passed!:" . $el->string if $el->string;
-            push @pieces, $self->blkstring(stop => $el->block);
+            push @pieces, $self->format_anchors($el), $self->blkstring(stop => $el->block);
         }
         elsif ($el->type eq 'regular') {
             # manage the special markup
             if ($el->string =~ m/\A\s*-----*\s*\z/s) {
-                push @pieces, $self->manage_hr($el);
+                push @pieces, $self->manage_hr($el), $self->format_anchors($el);
             }
             # an image by itself, so avoid it wrapping with <p></p>,
             # but only if just 1 is found. With multiple one, we get
@@ -173,20 +175,19 @@ sub process {
             elsif ($el->string =~ m/\A\s*\[\[\s*$imagere\s*\]
                                     (\[[^\]\[]+?\])?\]\s*\z/sx and
                    $el->string !~ m/\[\[.*\[\[/s) {
-                push @pieces, $self->manage_regular($el);
+                push @pieces, $self->format_anchors($el), $self->manage_regular($el);
             }
             else {
                 push @pieces, $self->manage_paragraph($el);
             }
         }
         elsif ($el->type eq 'standalone') {
-            push @pieces, $self->manage_regular($el, noanchors => 1);
+            push @pieces, $self->manage_regular($el);
         }
         elsif ($el->type eq 'dt') {
-            push @pieces, $self->manage_regular($el, noanchors => 1);
+            push @pieces, $self->manage_regular($el);
         }
-        elsif ($el->type =~ m/h[1-6]/) {
-
+        elsif ($el->is_header) {
             # if we want a split html, we cut here and flush the footnotes
             if ($el->type =~ m/h[1-4]/ and $split and @pieces) {
                 
@@ -209,7 +210,7 @@ sub process {
             push @pieces, $self->manage_header($el);
         }
         elsif ($el->type eq 'verse') {
-            push @pieces, $self->manage_verse($el);
+            push @pieces, $self->format_anchors($el), $self->manage_verse($el);
         }
         elsif ($el->type eq 'inlinecomment') {
             push @pieces, $self->manage_inline_comment($el);
@@ -218,13 +219,13 @@ sub process {
             push @pieces, $self->manage_comment($el);
         }
         elsif ($el->type eq 'table') {
-            push @pieces, $self->manage_table($el);
+            push @pieces, $self->format_anchors($el), $self->manage_table($el);
         }
         elsif ($el->type eq 'example') {
-            push @pieces, $self->manage_example($el);
+            push @pieces, $self->format_anchors($el), $self->manage_example($el);
         }
         elsif ($el->type eq 'newpage') {
-            push @pieces, $self->manage_newpage($el);
+            push @pieces, $self->manage_newpage($el), $self->format_anchors($el);
         }
         else {
             die "Unrecognized element: " . $el->type;
@@ -339,6 +340,7 @@ sub flush_secondary_footnotes {
 sub manage_html_footnote {
     my ($self, $element) = @_;
     return unless $element;
+    my $anchors = $self->format_anchors($element);
     my $fn_num = $element->footnote_index;
     my $fn_symbol = $element->footnote_symbol;
     my $class;
@@ -352,8 +354,8 @@ sub manage_html_footnote {
         die "wrong type " . $element->type . '  ' . $element->string;
     }
     my $chunk = qq{\n<p class="$class"><a class="footnotebody"} . " "
-      . qq{href="#fn_back${fn_num}" id="fn${fn_num}">$fn_symbol</a> } .
-        $self->manage_regular($element, noanchors => 1) .
+      . qq{href="#fn_back${fn_num}" id="fn${fn_num}">$fn_symbol</a>$anchors }
+      . $self->manage_regular($element) .
           qq{</p>\n};
 }
 
@@ -448,7 +450,6 @@ sub inline_elements {
                             ) |
                             (?<nobreakspace>  \~\~         ) |
                             (?<inline>(?:\*\*\*|\*\*|\*)   ) |
-                            (?<anchor> ^\x{20}* \#[A-Za-z][A-Za-z0-9]+\x{20}*(?:\n|$)) |
                             (?<br> \x{20}*\< br \x{20}* \/?\>)
                         )}gcxms) {
         # this is a mammuth, but hey
@@ -690,7 +691,7 @@ sub manage_regular {
     }
 
     # now we're hopefully set.
-    my (@out, @anchors);
+    my @out;
   CHUNK:
     while (@pieces) {
         my $piece = shift @pieces;
@@ -729,31 +730,17 @@ sub manage_regular {
                 $piece->type('text');
             }
         }
-        elsif ($piece->type eq 'anchor') {
-            # explicitely ignored
-            if ($opts{noanchors}) {
-                $piece->type('text');
-            }
-            else {
-                push @anchors, $piece->stringify;
-                next CHUNK;
-            }
-        }
         push @out, $piece->stringify;
     }
-    if ($opts{anchors}) {
-        return join('', @out), join('', @anchors);
-    }
-    else {
-        return join('', @out);
-    }
+    return join('', @out);
 }
 
 sub _format_footnote {
     my ($self, $element) = @_;
     if ($self->is_latex) {
         # print "Calling manage_regular from format_footnote " . Dumper($element);
-        my $footnote = $self->manage_regular($element, noanchors => 1);
+        my $footnote = $self->manage_regular($element);
+        my $anchors = $self->format_anchors($element);
         $footnote =~ s/\s+/ /gs;
         $footnote =~ s/ +$//s;
         # covert <br> to \par in latex. those \\ in the footnotes are
@@ -764,10 +751,10 @@ sub _format_footnote {
         # https://tex.stackexchange.com/questions/248620/footnote-of-several-paragraphs-length-to-section-title
         $footnote =~ s/\\forcelinebreak /\\protect\\endgraf /g;
         if ($element->type eq 'secondary_footnote') {
-            return '\footnoteB{' . $footnote . '}';
+            return '\footnoteB{' . $anchors . $footnote . '}';
         }
         else {
-            return '\footnote{' . $footnote . '}';
+            return '\footnote{' . $anchors . $footnote . '}';
         }
     } elsif ($self->is_html) {
         # in html, just remember the number
@@ -805,11 +792,9 @@ sub safe {
 
 sub manage_paragraph {
     my ($self, $el) = @_;
-    my ($body, $anchors) = $self->manage_regular($el, anchors => 1);
+    my $body = $self->manage_regular($el);
     chomp $body;
-    return $self->blkstring(start  => "p") .
-      $anchors .
-      $body . $self->blkstring(stop => "p");
+    return $self->blkstring(start  => "p") . $self->format_anchors($el) . $body . $self->blkstring(stop => "p");
 }
 
 =head3 manage_header
@@ -818,6 +803,7 @@ sub manage_paragraph {
 
 sub manage_header {
     my ($self, $el) = @_;
+    # print Dumper([$el->anchors]);
     my $body_with_no_footnotes = $el->string;
     my $has_fn;
     my $catch_fn = sub {
@@ -835,11 +821,12 @@ sub manage_header {
                                  )
                                 /$catch_fn->($1)/gxe;
     undef $catch_fn;
+    my $anchors = $self->format_anchors($el);
     my ($body_for_toc);
     if ($has_fn) {
-        ($body_for_toc) = $self->manage_regular($body_with_no_footnotes, nolinks => 1, anchors => 1);
+        ($body_for_toc) = $self->manage_regular($body_with_no_footnotes, nolinks => 1);
     }
-    my ($body, $anchors) = $self->manage_regular($el, nolinks => 1, anchors => 1);
+    my ($body) = $self->manage_regular($el, nolinks => 1);
     chomp $body;
     if (defined $body_for_toc) {
         $body_for_toc =~ s/\s+/ /g;
@@ -974,29 +961,26 @@ sub manage_verse {
     else { die "Not reached" }
 
     my (@chunks) = split(/\n/, $el->string);
-    my (@out, @stanza, @anchors);
+    my (@out, @stanza);
     foreach my $l (@chunks) {
         if ($l =~ m/\A( *)(.+?)\z/s) {
             my $leading = $lead x length($1);
-            my ($text, $anchors) = $self->manage_regular($2, anchors => 1);
-            if ($anchors) {
-                push @anchors, $anchors;
-            }
+            my $text = $self->manage_regular($2);
             if (length($text)) {
                 push @stanza, $leading . $text;
             }
         }
         elsif ($l =~ m/\A\s*\z/s) {
-            push @out, $self->_format_stanza(\@stanza, \@anchors);
-            die "wtf" if @stanza || @anchors;
+            push @out, $self->_format_stanza(\@stanza);
+            die "wtf" if @stanza;
         }
         else {
             die "wtf?";
         }
     }
-    # flush the stanzas and the anchors
-    push @out, $self->_format_stanza(\@stanza, \@anchors) if @stanza || @anchors;
-    die "wtf" if @stanza || @anchors;
+    # flush the stanzas
+    push @out, $self->_format_stanza(\@stanza) if @stanza;
+    die "wtf" if @stanza;
 
     # process
     return $self->blkstring(start => $el->type) .
@@ -1004,7 +988,7 @@ sub manage_verse {
 }
 
 sub _format_stanza {
-    my ($self, $stanza, $anchors) = @_;
+    my ($self, $stanza) = @_;
 
     my $eol;
     if ($self->is_html) {
@@ -1015,16 +999,12 @@ sub _format_stanza {
     }
     else { die "Not reached" };
 
-    my ($anchor_string, $stanza_string) = ('', '');
-    if (@$anchors) {
-        $anchor_string = join("\n", @$anchors);
-        @$anchors = ();
-    }
+    my $stanza_string = '';
     if (@$stanza) {
         $stanza_string = join($eol, @$stanza);
         @$stanza = ();
     }
-    return $anchor_string . $stanza_string;
+    return $stanza_string;
 }
 
 
@@ -1166,7 +1146,7 @@ sub manage_table_ltx {
     $textable .= "\\hline\n\\end{tabularx}\n";
     if ($has_caption) {
         $textable .= "\n\\caption[]{" .
-          $self->manage_regular($table->{caption}, noanchors => 1)
+          $self->manage_regular($table->{caption})
           . "}\n";
     }
     $textable .= "\\end{minipage}\n";
@@ -1320,7 +1300,7 @@ sub linkify {
 
 sub format_links {
     my ($self, $link, $desc) = @_;
-    $desc = $self->manage_regular($desc, noanchors => 1);
+    $desc = $self->manage_regular($desc);
     # first the images
     if (my $image = $self->find_image($link)) {
         my $src = $image->filename;
@@ -1329,7 +1309,7 @@ sub format_links {
         return $image->output;
     }
     # links
-    if ($link =~ m/\A\#([A-Za-z][A-Za-z0-9]*)\z/) {
+    if ($link =~ m/\A\#([A-Za-z][A-Za-z0-9-]*)\z/) {
         my $linkname = $1;
         if ($self->is_html) {
             $link = "#text-amuse-label-$linkname";
@@ -1859,4 +1839,21 @@ sub _latex_header {
     }
 }
 
+=head3 format_anchors($element)
+
+Return a formatted string with the anchors found in the element.
+
+=cut
+
+sub format_anchors {
+    my ($self, $el) = @_;
+    my $out = '';
+    if (my @anchors = map { Text::Amuse::InlineElement->new(string => $_,
+                                                            type => 'anchor',
+                                                            fmt => $self->fmt)->stringify } $el->anchors) {
+        return join('', @anchors);
+    }
+    return $out;
+}
+
 1;
